/*
 * callbacks.c -- Callbacks for fireswamp
 * Copyright (C) 2004-2005 Charles Yates
 * Author: Charles Yates <charles.yates@pandora.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkcolor.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <framework/mlt.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

static GtkWidget *window = NULL;
static mlt_playlist playlist = NULL;
static mlt_properties producers = NULL;
static mlt_properties producers_old = NULL;
static mlt_consumer consumer = NULL;
static mlt_position current_clip = 0;
static mlt_position in_position = 0;
static mlt_position out_position = 0;
static mlt_position current_position = 0;
static int terminated = 0;
static int locked = 0;
static pthread_t thread;
static char *last_dir = NULL;
static char last_resource[ 2048 ] = "";
static int alwaysontop = 0;
static int fullscreen = 0;

int toggle_alwaysontop( void )
{
	XEvent ev;
	Display *display = GDK_WINDOW_XDISPLAY( window->window );
	Atom net_wm_state = gdk_x11_get_xatom_by_name( "_NET_WM_STATE" );
	Atom net_wm_state_above = gdk_x11_get_xatom_by_name( "_NET_WM_STATE_ABOVE" );

	alwaysontop = !alwaysontop;

	ev.type = ClientMessage;
	ev.xclient.window = GDK_WINDOW_XWINDOW( window->window );
	ev.xclient.message_type = net_wm_state;
	ev.xclient.format = 32;
	ev.xclient.data.l[ 0 ] = alwaysontop ? 1 : 0;
	ev.xclient.data.l[ 1 ] = net_wm_state_above;
	ev.xclient.data.l[ 2 ] = 0;

	XSendEvent( display, DefaultRootWindow( display ), False,
				SubstructureNotifyMask|SubstructureRedirectMask, &ev );

    return alwaysontop;
}

int toggle_fullscreen( void )
{
	XEvent ev;
	Display *display = GDK_WINDOW_XDISPLAY( window->window );
	Atom net_wm_state = gdk_x11_get_xatom_by_name( "_NET_WM_STATE" );
	Atom net_wm_state_fullscreen = gdk_x11_get_xatom_by_name( "_NET_WM_STATE_FULLSCREEN" );

	fullscreen = !fullscreen;

	ev.type = ClientMessage;
	ev.xclient.window = GDK_WINDOW_XWINDOW( window->window );
	ev.xclient.message_type = net_wm_state;
	ev.xclient.format = 32;
	ev.xclient.data.l[ 0 ] = fullscreen ? 1 : 0;
	ev.xclient.data.l[ 1 ] = net_wm_state_fullscreen;
	ev.xclient.data.l[ 2 ] = 0;

	XSendEvent( display, DefaultRootWindow( display ), False,
				SubstructureNotifyMask|SubstructureRedirectMask, &ev );

    return fullscreen;
}


static void shutdown( )
{
	gdk_threads_leave();
	terminated = 1;
	pthread_join( thread, NULL );
	mlt_consumer_close( consumer );
	mlt_playlist_close( playlist );
	mlt_properties_close( producers_old );
	mlt_properties_close( producers );
	mlt_factory_close( );
	gdk_threads_enter();
	gtk_exit( 0 );
}

/** Window close event.
*/

static gboolean on_main_window_delete_event( GtkWidget *widget, GdkEvent *event, gpointer user_data )
{
	shutdown( );
	return TRUE;
}

static void show_status( )
{
	gdk_threads_enter();
	if ( !locked )
	{
		GtkRange *range = GTK_RANGE( lookup_widget( window, "hscale" ) );
		GtkAdjustment *adjustment = gtk_range_get_adjustment( range );
		int length = out_position + 1;
		if ( length > 0 && adjustment->upper != length )
			gtk_range_set_range( range, 0, length );
		gtk_range_set_value( range, current_position );
	}
	gdk_threads_leave();
}

static void *status_thread( void *arg )
{
	struct timespec tm = { 0, 200000000 };
	mlt_playlist_clip_info info;
	mlt_properties properties = NULL;
	char *resource = NULL;
	
	while( !terminated )
	{
		current_clip = mlt_playlist_current_clip( playlist );
		mlt_playlist_get_clip_info( playlist, &info, current_clip );
		in_position = info.start;
		out_position = info.frame_out;
		current_position = mlt_producer_position( mlt_playlist_producer( playlist ) ) - in_position;
		if ( info.producer != NULL )
		{
			properties = mlt_producer_properties( info.producer );
			resource = mlt_properties_get( properties, "resource" );
			if ( strcmp( last_resource, resource ) )
			{
				GtkWidget *treeview = lookup_widget( window, "treeview1" );
				GtkListStore *liststore = GTK_LIST_STORE( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ) ) );
				GtkTreeIter iter;
				gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(liststore), &iter);
				int index = 0;

				while ( valid )
				{
					gtk_list_store_set(liststore, &iter, 1, index == current_clip ? "red" : "white", -1);
					valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(liststore), &iter);
					index ++;
				}

				strcpy( last_resource, resource );
				if ( strchr( resource, '/' ) )
					gtk_window_set_title( GTK_WINDOW( window ), strrchr( resource, '/' ) + 1 );
				else
					gtk_window_set_title( GTK_WINDOW( window ), resource );
			}
		}
		else if ( strcmp( last_resource, "Fireswamp" ) )
		{
			strcpy( last_resource, "Fireswamp" );
			gtk_window_set_title( GTK_WINDOW( window ), last_resource );
		}
		show_status( );
		nanosleep( &tm, NULL );
	}

	return NULL;
}

static void list_clips( char *file )
{
	GtkWidget *treeview = lookup_widget( window, "treeview1" );
	if ( file != NULL )
	{
		GtkListStore *clip_store = NULL;
		GtkTreeIter iter;
		int index;

		if (  gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ) ) == NULL )
		{
			GtkCellRenderer *renderer;
			GtkTreeViewColumn *column;

			clip_store = gtk_list_store_new( 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING );
			gtk_tree_view_set_model( GTK_TREE_VIEW( treeview ), GTK_TREE_MODEL( clip_store ) );
			
			renderer = gtk_cell_renderer_text_new( );
			column = gtk_tree_view_column_new_with_attributes( "Description", renderer, "text", 0, "foreground", 1, "background", 2, NULL);
			gtk_tree_view_column_set_sort_column_id( column, 0 );
			gtk_tree_view_append_column( GTK_TREE_VIEW( treeview ), column );
		}
		else
		{
			clip_store = GTK_LIST_STORE( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ) ) );
		}
		
		gtk_list_store_append( clip_store, &iter );
		if ( strchr( file, '/' ) != NULL )
			gtk_list_store_set( clip_store, &iter, 0, strrchr( file, '/' ) + 1, 1, "white", 2, "black", -1 );
		else
			gtk_list_store_set( clip_store, &iter, 0, file, 1, "white", 2, "black", -1 );
	}
	else
	{
		if ( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ) ) )
		{
			GtkListStore *list_store = GTK_LIST_STORE( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ) ) );
			gtk_list_store_clear( list_store );
		}
	}
}


static void playlist_clear( )
{
	mlt_playlist_clear( playlist );
	list_clips( NULL );

	if ( producers_old )
		mlt_properties_close( producers_old );
	producers_old = producers;
	producers = mlt_properties_new( );
	mlt_producer_seek( mlt_playlist_producer( playlist ), 0 );
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
}

static void playlist_recover( )
{
	if ( producers_old )
	{
		int i = 0;
		for ( i = 0; i < mlt_properties_count( producers_old ); i ++ )
		{
			mlt_producer ptr = mlt_properties_get_data_at( producers_old, i, NULL );
			if ( ptr != NULL )
			{
				mlt_playlist_append( playlist, ptr );
				list_clips( mlt_properties_get( mlt_producer_properties( ptr ), "resource" ) );
			}
		}
		producers = producers_old;
		producers_old = NULL;
	}
}

static void playlist_add( char *file )
{
	char label[ 256 ];
	mlt_producer producer = mlt_factory_producer( NULL, file );
	if ( producer != NULL )
	{
		sprintf( label, "%x", producer );
		mlt_properties_set_data( producers, label, producer, 0, ( mlt_destructor )mlt_producer_close, NULL );
		mlt_playlist_append( playlist, producer );
		list_clips( file );
	}
	if ( strchr( file, '/' ) != NULL )
	{
		free( last_dir );
		last_dir = strdup( file );
		*( strrchr( last_dir, '/' ) + 1 ) = '\0';
	}

	while ( gtk_events_pending() )
		gtk_main_iteration();
}


int run( int argc, char **argv )
{
	static GtkTargetEntry file_targets[] = { {"text/uri-list", 0, 0} };

	if ( mlt_factory_init( NULL ) == 0 )
	{
		window = create_fireswamp( );
    	gtk_signal_connect( GTK_OBJECT( window ), "destroy", GTK_SIGNAL_FUNC( on_main_window_delete_event ), NULL );
    	gtk_widget_show( window );

		GtkWidget *drawingarea = lookup_widget( window, "drawingarea" );
		GtkWidget *treeview = lookup_widget( window, "treeview1" );
		gtk_widget_set_double_buffered( lookup_widget( window, "drawingarea" ), FALSE );

		gtk_drag_dest_set( drawingarea, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP, file_targets, 1, GDK_ACTION_COPY | GDK_ACTION_MOVE );
		gtk_drag_dest_set( treeview, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP, file_targets, 1, GDK_ACTION_COPY | GDK_ACTION_MOVE );

		Window xwin = GDK_WINDOW_XWINDOW( lookup_widget( window, "drawingarea" )->window );
		gchar SDL_windowhack[32];
		sprintf( SDL_windowhack, "SDL_WINDOWID=%ld", xwin );
		putenv( SDL_windowhack );

		GdkColor colour;
		gdk_color_black( gdk_colormap_get_system( ), &colour );
		gtk_widget_modify_base( treeview, GTK_STATE_NORMAL, &colour );

		gdk_threads_enter();

		consumer = mlt_factory_consumer( "sdl", NULL );
		mlt_properties_set( mlt_consumer_properties( consumer ), "rescale", "none" );
		mlt_properties_set( mlt_consumer_properties( consumer ), "progressive", "1" );
		mlt_properties_set( mlt_consumer_properties( consumer ), "audio_buffer", "1024" );
		mlt_properties_set( mlt_consumer_properties( consumer ), "frequency", "44100" );
		mlt_properties_set( mlt_consumer_properties( consumer ), "buffer", "1" );
		mlt_properties_set( mlt_consumer_properties( consumer ), "resize", "1" );

		playlist = mlt_playlist_init( );
		producers = mlt_properties_new( );

		if ( argc > 1 )
		{
			int i;
			for ( i = 1; i < argc; i ++ )
				playlist_add( argv[ i ] );
		}

		mlt_consumer_connect( consumer, mlt_playlist_service( playlist ) );
		mlt_consumer_start( consumer );

		pthread_create( &thread, NULL, status_thread, NULL );
    
		gtk_main();
		terminated = 1;
		pthread_join( thread, NULL );
		gdk_threads_leave();
	}
	else
	{
		fprintf( stderr, "Unable to create mlt factory\n" );
	}

	return 0;
}

int tohex( char p )
{
	return isdigit( p ) ? p - '0' : tolower( p ) - 'a' + 10;
}

char *url_decode( char *dest, char *src )
{
	char *p = dest;
	
	while ( *src )
	{
		if ( *src == '%' )
		{
			*p ++ = ( tohex( *( src + 1 ) ) << 4 ) | tohex( *( src + 2 ) );
			src += 3;
		}
		else
		{
			*p ++ = *src ++;
		}
	}

	*p = *src;

	return dest;
}

void
on_button0_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_seek( mlt_playlist_producer( playlist ), in_position );
}


void
on_button1_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), -50 );
}


void
on_button2_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), 0 );
	mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + current_position - 1 );
}


void
on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	if ( mlt_playlist_count( playlist ) == 0 )
		playlist_recover( );
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
}


void
on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), 0 );
	mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + current_position + 1 );
}


void
on_button5_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_set_speed( mlt_playlist_producer( playlist ), 50 );
}


void
on_button6_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + out_position + 1 );
}


gboolean
on_hscale_button_press_event           (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	locked = 1;
  	return FALSE;
}


gboolean
on_hscale_button_release_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	GtkAdjustment *adjustment = gtk_range_get_adjustment( GTK_RANGE( widget ) );
	mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + adjustment->value );
	locked = 0;
  	return FALSE;
}


void
on_hscale_value_changed                (GtkRange        *range,
                                        gpointer         user_data)
{
	if ( locked )
	{
		GtkAdjustment *adjustment = gtk_range_get_adjustment( range );
		mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + adjustment->value );
	}
}


gboolean
on_hscale_key_press_event              (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	switch ( event->keyval )
	{
		case GDK_Left:
		case GDK_Right:
		case GDK_Page_Up:
		case GDK_Page_Down:
			locked = 1;
			mlt_producer_set_speed( mlt_playlist_producer( playlist ), 0 );
  			break;
		case GDK_Home:
			mlt_producer_seek( mlt_playlist_producer( playlist ), in_position );
			break;
		case GDK_End:
		case GDK_Down:
			mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + out_position + 1 );
			break;
		case GDK_Up:
			mlt_producer_seek( mlt_playlist_producer( playlist ), mlt_playlist_clip( playlist,mlt_whence_relative_current, -1 ) );
			break;
	}
  	return FALSE;
}


gboolean
on_hscale_key_release_event            (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	if ( locked )
	{
		GtkAdjustment *adjustment = gtk_range_get_adjustment( GTK_RANGE( widget ) );
		mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + adjustment->value );
		mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
		locked = 0;
	}
  	return FALSE;
}


gboolean
on_fireswamp_key_press_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	GtkWidget *hscale = lookup_widget( widget, "hscale" );
	GtkWidget *treeview = lookup_widget( widget, "treeview1" );

	switch ( event->keyval )
	{
		case GDK_Left:
		case GDK_Right:
		case GDK_Page_Up:
		case GDK_Page_Down:
			if ( !GTK_WIDGET_HAS_FOCUS( hscale ) )
			{
				locked = 1;
				gtk_widget_grab_focus( hscale );
				mlt_producer_set_speed( mlt_playlist_producer( playlist ), 0 );
			}
  			break;
		case GDK_Home:
			mlt_producer_seek( mlt_playlist_producer( playlist ), in_position );
			break;
		case GDK_End:
		case GDK_Down:
			if ( !GTK_WIDGET_HAS_FOCUS( hscale ) && !GTK_WIDGET_HAS_FOCUS( treeview ) )
				mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + out_position + 1 );
			break;
		case GDK_Up:
			if ( !GTK_WIDGET_HAS_FOCUS( hscale ) && !GTK_WIDGET_HAS_FOCUS( treeview ) )
				mlt_producer_seek( mlt_playlist_producer( playlist ), mlt_playlist_clip( playlist,mlt_whence_relative_current, -1 ) );
			break;
		case 'q':
			shutdown( );
			return TRUE;
			break;
		case ' ':
			mlt_producer_set_speed( mlt_playlist_producer( playlist ), mlt_producer_get_speed( mlt_playlist_producer( playlist ) ) == 0 ? 1 : 0 );
			return TRUE;
		case 'f':
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget( window, "togglebutton3" ) ), !fullscreen );
			return TRUE;
	}

  	return FALSE;
}


void
on_button7_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *dialog = gtk_file_selection_new( "Select your ROUS" );

	gtk_file_selection_set_select_multiple( GTK_FILE_SELECTION( dialog ), TRUE );

	if ( last_dir != NULL )
		gtk_file_selection_set_filename( GTK_FILE_SELECTION( dialog ), last_dir );

	if ( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_OK )
  	{
		gchar **files = gtk_file_selection_get_selections( GTK_FILE_SELECTION( dialog ) );
		int i = 0;

		gtk_widget_destroy (dialog);

		playlist_clear( );

		while( files[ i ] != NULL )
		{
    		playlist_add( files[ i ++ ] );
			mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
		}

		g_strfreev( files );
  	}
	else
	{
		gtk_widget_destroy (dialog);
	}
}

gboolean
on_fireswamp_focus_out_event           (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
	locked = 0;
  	return FALSE;
}


gboolean
on_drawingarea_expose_event            (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	mlt_properties_set_int( mlt_consumer_properties( consumer ), "changed", 1 );
  	return FALSE;
}



void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	playlist_clear( );
}


void
on_togglebutton1_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	mlt_consumer_stop( consumer );
	if ( gtk_toggle_button_get_active( togglebutton ) )
		mlt_properties_set( mlt_consumer_properties( consumer ), "rescale", "nearest" );
	else
		mlt_properties_set( mlt_consumer_properties( consumer ), "rescale", "none" );
	mlt_consumer_start( consumer );
}


void
on_treeview1_drag_data_received        (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
	if ( data->data )
	{
		mlt_tokeniser tokeniser = mlt_tokeniser_init( );
		char *arg;
		char temp[ 1024 ];
		int i = 0;

		mlt_tokeniser_parse_new( tokeniser, data->data, "\n" );

		for ( i = 0; i < mlt_tokeniser_count( tokeniser ); i ++ )
		{
			char *tmp = mlt_tokeniser_get_string( tokeniser, i );
			if ( !strncmp( tmp, "file:/", 6 ) )
			{
				char label[ 256 ];
				arg = tmp + 5;
				while ( arg[ strlen( arg ) - 1 ] == '\n' || arg[ strlen( arg ) - 1 ] == '\r' )
					arg[ strlen( arg ) - 1 ] = '\0';
				playlist_add( url_decode( temp, arg ) );
				mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
			}
		}
		mlt_tokeniser_close( tokeniser );
	}

}


void
on_drawingarea_drag_data_received      (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
	if ( data->data )
	{
		mlt_tokeniser tokeniser = mlt_tokeniser_init( );
		char *arg;
		char temp[ 1024 ];
		int i = 0;

		mlt_tokeniser_parse_new( tokeniser, data->data, "\n" );
		playlist_clear( );

		for ( i = 0; i < mlt_tokeniser_count( tokeniser ); i ++ )
		{
			char *tmp = mlt_tokeniser_get_string( tokeniser, i );
			if ( !strncmp( tmp, "file:/", 6 ) )
			{
				char label[ 256 ];
				arg = tmp + 5;
				while ( arg[ strlen( arg ) - 1 ] == '\n' || arg[ strlen( arg ) - 1 ] == '\r' )
					arg[ strlen( arg ) - 1 ] = '\0';
				playlist_add( url_decode( temp, arg ) );
				mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
			}
		}
		mlt_tokeniser_close( tokeniser );
	}

}

void
on_treeview1_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	GtkWidget *widget = lookup_widget( window, "treeview1" );
	GtkTreeSelection *select = gtk_tree_view_get_selection( GTK_TREE_VIEW( widget ) );
	GtkTreeModel *model;
	GtkTreeIter iter;
	GValue value;

	if ( gtk_tree_selection_get_selected( select, &model, &iter ) )
	{
		GtkTreePath *path = gtk_tree_model_get_path( model, &iter );
		int clip = atoi( gtk_tree_path_to_string( path ) );
		mlt_producer_seek( mlt_playlist_producer( playlist ), mlt_playlist_clip( playlist,mlt_whence_relative_start, clip ) );
		mlt_producer_set_speed( mlt_playlist_producer( playlist ), 1 );
	}
}


void
on_togglebutton2_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	toggle_alwaysontop( );
}


void
on_togglebutton3_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *widget = lookup_widget( window, "hbox1" );
	toggle_fullscreen( );
	if ( fullscreen )
	{
		gtk_widget_hide_all( lookup_widget( window, "hbox1" ) );
		gtk_widget_hide_all( lookup_widget( window, "hscale" ) );
	}
	else
	{
		gtk_widget_show_all( lookup_widget( window, "hbox1" ) );
		gtk_widget_show_all( lookup_widget( window, "hscale" ) );
	}
}


gboolean
on_fireswamp_key_release_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_fireswamp_scroll_event              (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	//if ( event->scroll.direction == GDK_SCROLL_UP )
		//mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + current_position - 10 );
	//else if ( event->scroll.direction == GDK_SCROLL_DOWN )
		//mlt_producer_seek( mlt_playlist_producer( playlist ), in_position + current_position + 10 );
  	return FALSE;
}

